#include "drawwidget.h"
#include <QKeyEvent>
#include <QMouseEvent>
#include <pipeline.h>
#include <shadersource.h>
#include <framebuffer.h>
#include <resource.h>
#include <glm/ext.hpp>
#include "mapscene.h"
#include "pixelpipeline.h"
#include <QCoreApplication>

DrawWidget::DrawWidget(QWidget *parent) :
    QOpenGLWidget(parent),
    m_Qt_fbo(NULL)
{
    new_simulation = false;
    update_needed = false;
}

DrawWidget::~DrawWidget()
{
    delete m_pipeline;
    delete m_map;
    if(m_Qt_fbo != NULL && m_Qt_fbo != FrameBuffer::screen)
        delete(m_Qt_fbo);
}

void DrawWidget::initializeGL()
{
    renderer.initGL(width(), height());
    m_map = NULL;
    renderer.setScene(m_map);
}

void DrawWidget::paintGL()
{
    if(new_simulation)
    {
        m_pipeline = new PixelPipeline(m_map);
        m_pipeline->setTargetFBO(m_Qt_fbo);
        m_pipeline->resizeGL(m_width, m_height);
        renderer.setScene(m_map);
        new_simulation = false;
    }
    else if(update_needed)
        m_pipeline->updateChanges();
    renderer.renderGL();
}

void DrawWidget::updateDudesBehavior()
{
    update_needed = true;
}

void DrawWidget::resizeGL(int w, int h)
{
    m_width = w;
    m_height = h;
    renderer.resizeGL(w, h);
    if(m_Qt_fbo != NULL && m_Qt_fbo != FrameBuffer::screen)
        delete(m_Qt_fbo);
    m_Qt_fbo = new FrameBuffer(defaultFramebufferObject());
    if(m_map != NULL)
        m_pipeline->setTargetFBO(m_Qt_fbo);
}

void DrawWidget::startSimulation(GenerateFunction genFunc, std::vector<BehaviorFunction> behaveFuncs)
{
    if(m_map != NULL)
    {
        delete m_map;
        delete m_pipeline;
        m_map = NULL;
    }
    m_map = new MapScene(behaveFuncs.size(), 300);
    genFunc((Map*)m_map);
    new_simulation = true;
    repaint();
}

// INPUT EVENTS

void DrawWidget::mouseMoveEvent(QMouseEvent *event)
{
    if(grabbedMouse)
    {
        m_pipeline->cameraMove(event->globalX() - lastMousePos.x(), event->globalY() - lastMousePos.y());
        repaint();
    }
    lastMousePos = event->globalPos();
}

void DrawWidget::mousePressEvent(QMouseEvent* event)
{
    switch(event->button())
    {
        case Qt::LeftButton :
            grabbedMouse = true;
            break;
        default:
            break;
    }
    lastMousePos = event->globalPos();
}

void DrawWidget::mouseDoubleClickEvent(QMouseEvent * event)
{
    
}

void DrawWidget::mouseReleaseEvent(QMouseEvent* event)
{
    switch(event->button())
    {
        case Qt::LeftButton :
            grabbedMouse = false;
            break;
        default:
            break;
    }
}

void DrawWidget::wheelEvent(QWheelEvent *event)
{
    m_pipeline->cameraZoom(event->delta());
    repaint();
}
